--[[
	Author:  Azmaedus US-Arathor
	Version: 0.3.1b
	compatible with game version 6.0.3
	
	Change Log
	----------
	11/29/2014 - 0.1 	Creation of addon
	11/29/2014 - 0.1.1 	Support for tier 1, 2, and 3 garrison (Outpost, Fort, and Garrison)
	12/01/2014 - r8		1st commit to wowace.com (alpha)
	12/02/2014 - r9		Added multi-dimensional table to hold music filename references
	12/26/2014 - r9		Fixed issue with music not playing inside garrison buildings by changing the way to detect if we are inside Lunarfall
	12/27/2014 - r10	2 minute timer added between random playlist items.
						Dropdown music now functional per expansion selected.
						Fixed reload (musiclist) not loading in profile; forgot DB at the end of savedvars .toc
	12/28/2014 - r11	Fixed /reload issue not playing music
	12/29/2014 - 0.2b	1st beta: Fixed error when opening the options UI
	12/29/2014 - 0.3b	Changed from using 2 table design to a single table for holding music information
	12/29/2014 - 0.3.1b	Added Evil Forest 1, 2 and 3 music (Silverpine & Tirisfal)
	
	Known Bugs
	----------
	- typing /gjb for the first time during a character login session or after /reload does not bring up the addon options UI
	
]]

-- Load the Ace libraries --
GarrisonJukeBox = LibStub("AceAddon-3.0"):NewAddon("GarrisonJukeBox", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GarrisonJukeBox")

-- globals
local gVersion = "0.3.1b"
local gMusicPlaying = false 	-- is the music currently playing
local gMusicIndex = 1			-- current item index of table musicfiles
local gMaxMusic = 0				-- number of items in musicfiles table
local gOnLoadMusicTimer = 5		-- HACK!!! wait n seconds when PLAYYER_LOGIN is triggered before playing music (make sure Blizz level loading script doesn't override PlayMusic() call)
local gPlayerLogin = false		-- used to acknowledge that a player has logged in or reloaded ui
local LUNARFALL = 971			-- zone id of Lunarfall

-- temp storage for select vars
local expac = "i1wow"
local music = 1
local jukebox = ""

-- Create the default settings for this addon --
local defaults = { 
    profile =  {
		musiclist = {},
		shuffle = true,
	},
}

-- table holding music info (name, file, length)
local musictable = {
	-- Classic
	i1wow = {
		{"Main Theme", "Sound\\Music\\GlueScreenMusic\\wow_main_theme.mp3", 161},
		{"Darnassus", "Sound\\Music\\CityMusic\\Darnassus\\Darnassus Intro.mp3", 120},
		{"Druid Grove", "Sound\\Music\\CityMusic\\Darnassus\\Druid Grove.mp3", 120},
		{"Warrior Terrace", "Sound\\Music\\CityMusic\\Darnassus\\Warrior Terrace.mp3", 120},
		{"IronForge", "Sound\\Music\\CityMusic\\Ironforge\\IronForge Intro.mp3", 120},
		{"Gnomeragon 1", "Sound\\Music\\CityMusic\\Gnomeragon\\gnomeragon01-zone.mp3", 120},
		{"Gnomeragon 2", "Sound\\Music\\CityMusic\\Gnomeragon\\gnomeragon02-zone.mp3", 120},
		{"Orgrimmar 1", "Sound\\Music\\CityMusic\\Orgrimmar\\orgrimmar01-zone.mp3", 120},
		{"Orgrimmar 2", "Sound\\Music\\CityMusic\\Orgrimmar\\orgrimmar02-zone.mp3", 120},
		{"Stormwind 1", "Sound\\Music\\CityMusic\\Stormwind\\stormwind04-zone.mp3", 120},
		{"Stormwind 2", "Sound\\Music\\CityMusic\\Stormwind\\stormwind05-zone.mp3", 120},
		{"Tavern Alliance 1", "Sound\\Music\\ZoneMusic\\TavernAlliance\\TavernAlliance01.mp3", 120},
		{"Tavern Alliance 2", "Sound\\Music\\ZoneMusic\\TavernAlliance\\TavernAlliance02.mp3", 120},
		{"Tavern Horde 1", "Sound\\Music\\ZoneMusic\\TavernHorde\\TavernHorde01.mp3", 120},
		{"Tavern Horde 2", "Sound\\Music\\ZoneMusic\\TavernHorde\\TavernHorde02.mp3", 120},
		{"Evil Forest 1", "Sound\\Music\\ZoneMusic\\EvilForest\\NightEvilForest01.mp3", 57},
		{"Evil Forest 2", "Sound\\Music\\ZoneMusic\\EvilForest\\NightEvilForest02.mp3", 75},
		{"Evil Forest 3", "Sound\\Music\\ZoneMusic\\EvilForest\\NightEvilForest03.mp3", 70},
	},

	-- The Burning Crusade
	i2bc = {
		{"Main Theme", "Sound\\Music\\GlueScreenMusic\\BC_main_theme.mp3", 226},
		{"Azuremyst", "Sound\\Music\\ZoneMusic\\Azuremyst\\AI_DraeneiWalkUni05.mp3", 120},
		{"Caverns of Time 1", "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_CavernsWalk1Uni.mp3", 120},
		{"Caverns of Time 2", "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_CavernsWalk2Uni.mp3", 120},
		{"Caverns of Time 3", "Sound\\Music\\ZoneMusic\\CavernsOfTime\\CT_CavernsWalk3Uni.mp3", 120},
		{"Outland", "Sound\\Music\\ZoneMusic\\OutlandGeneral\\OL_Corrupt01.mp3", 120},
	},
	
	-- Wrath of the Lich King
	i3wotlk = {
		{"Main Theme", "Sound\\Music\\GlueScreenMusic\\WotLK_main_title.mp3", 544},
	},
	
	-- Cataclysm
	i4cata = {
		{"Main Theme", "Sound\\Music\\cataclysm\\MUS_Shattering_UU01.mp3", 725},
	},
	
	-- Mists of Pandaria
	i5mop = {
		{"Main Theme", "Sound\\Music\\Pandaria\\MUS_50_HeartofPandaria_01.mp3", 480},
	},
	
	-- Warlords of Draenor
	i6wod = {
		{"Main Theme", "Sound\\Music\\Draenor\\MUS_60_ASiegeofWorlds_MainTitle.mp3", 726},
	},
	
	-- Warcraft III
	i7wc3 = {
		{"Power of the Horde", "Sound\\Music\\ZoneMusic\\DMF_L70ETC01.mp3", 282},
	},
}

-- Create the options variable to use in the config screen --
local options = {
	name = L["Garrison JukeBox Options"],
	desc = L["Build your playlist here."],
	handler = GarrisonJukeBox,
	type = "group",
	args = {
		expac = {
			type = "select",
			name = L["Expansion"],
			desc = L["Select the expansion."],
			style = "dropdown",
			values = "GetExpanssions",
			set = "SetExpac",
			get = "GetExpac",
			width = "normal",
			order = 1,
		},
		music = {
			type = "select",
			name = L["Music"],
			desc = L["Select the music."],
			style = "dropdown",
			values = "GetCurExpacMusicList",
			set = "SetMusic",
			get = "GetMusic",
			width = "normal",
			order = 2,
		},
		musicadd = {
			type = "execute",
			name = L["Add"],
			desc = L["Adds the music to your JukeBox list."],
			func = "AddMusic",
			width = "half",
			order = 3,
		},
		musiclist = {
			type = "select",
			name = L["Your JukeBox"],
			desc = L["Your custom music list."],
			style = "dropdown",
			values = "GetJukeBoxList",
			set = "SetMusicProfile",
			get = "GetMusicProfile",
			width = "double",
			order = 50,
		},
		musicdelete = {
			type = "execute",
			name = L["Remove"],
			desc = L["Removes the selected music from your JukeBox list."],
			func = "DelMusic",
			width = "half",
			order = 80,
		},
		musicdeleteall = {
			type = "execute",
			name = L["Remove All"],
			desc = L["Removes all music from your JukeBox list."],
			func = "DelMusicAll",
			width = "normal",
			order = 100,
		},
		shuffle = {
			type = "toggle",
			name = L["Shuffle"],
			desc = L["Randomly plays the music from your playlist."],

			set = function(info, value)
				GarrisonJukeBox.db.profile.shuffle = value
			end,

			get = function(info)
				return GarrisonJukeBox.db.profile.shuffle
			end,

			width = "full",
			order = 120,
		},
	},
}

function GarrisonJukeBox:GetExpanssions()
	return {
		i1wow = EXPANSION_NAME0,
		i2bc = EXPANSION_NAME1,
		i3wotlk = EXPANSION_NAME2,
		i4cata = EXPANSION_NAME3,
		i5mop = EXPANSION_NAME4,
		i6wod = EXPANSION_NAME5,
		i7wc3 = "Warcraft III",
	}
end

-- Expac functions
function GarrisonJukeBox:SetExpac(info, value)
	expac = value
end

function GarrisonJukeBox:GetExpac(info)
	return expac
end

-- Music functions
function GarrisonJukeBox:SetMusic(info, value)
	music = value
end

function GarrisonJukeBox:GetMusic(info)
	return music
end

function GarrisonJukeBox:GetCurExpacMusicList()
	local tbl = musictable[self:GetExpac()]
	local t = {}

	for k, v in ipairs(tbl) do
		table.insert(t, k, v[1])
	end
	
	return t
end

-- Add, Remove, Remove All music functions
function GarrisonJukeBox:AddMusic()
	local xps = self:GetExpanssions()
	local xp = xps[self:GetExpac()]
	local mus = self:GetCurExpacMusicList()
	local mu = mus[self:GetMusic()]
	
	-- TODO: Will we allow duplicates?
	table.insert(GarrisonJukeBox.db.profile.musiclist, xp .. "/" .. mu)
	jukebox = xp .. "/" .. mu
end

function GarrisonJukeBox:DelMusic()
	GarrisonJukeBox.db.profile.musiclist[jukebox] = nil
end

function GarrisonJukeBox:DelMusicAll()
	GarrisonJukeBox.db.profile.musiclist = {}
	jukebox = ""
end

-- Profile Music List
function GarrisonJukeBox:GetJukeBoxList()
	return GarrisonJukeBox.db.profile.musiclist
end

function GarrisonJukeBox:SetMusicProfile(info, value)
	jukebox = value
end

function GarrisonJukeBox:GetMusicProfile(info)
	return jukebox
end

-- OnInitialize
function GarrisonJukeBox:OnInitialize()
	GarrisonJukeBox.db = LibStub("AceDB-3.0"):New("GarrisonJukeBoxDB", defaults, true)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GarrisonJukeBoxDB", options)
	GarrisonJukeBox.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GarrisonJukeBoxDB", "Garrison JukeBox")

	self:RegisterChatCommand("gjb", "ChatCommand")

	gMusicIndex = 1
	gMusicPlaying = false
	gMaxMusic = 0
	gLoaded = false
	self.musicTimer = nil
	
	self:Print(L["Garrison JukeBox"] .. " " .. gVersion .. " " .. L["initialized!"])
end

-- OnLoad
function GarrisonJukeBox:OnLoad()
	GarrisonJukeBox:RegisterEvent("PLAYER_LOGIN", "OnLoginEvent")
	GarrisonJukeBox:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPEWEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED", "OnZoneEvent")
		
	self:Print(L["Garrison JukeBox"] .. " " .. gVersion .. " " .. L["loaded!"])
end

-- Events
function GarrisonJukeBox:OnLoginEvent(event, ...)
	StopMusic()
	gMusicPlaying = false
	gPlayerLogin = true
	self.musicTimer = self:ScheduleTimer("TimerEvent", gOnLoadMusicTimer)
end

function GarrisonJukeBox:OnPEWEvent(event, ...)
	if gPlayerLogin == false then
		StopMusic()
		gMusicPlaying = false
		self.musicTimer = self:ScheduleTimer("TimerEvent", gOnLoadMusicTimer)
	end
end

function GarrisonJukeBox:OnZoneEvent(event, ...)
	self:ProcessInfo()
end

-- Ace3 Timer event for playing music
function GarrisonJukeBox:TimerEvent()
	self:ProcessInfo()
	gPlayerLogin = false
end

-- function called to determine if music needs to be played after a registered event has fired
function GarrisonJukeBox:ProcessInfo()
	if GetCurrentMapAreaID() == LUNARFALL then
		if gMusicPlaying == false then
			gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
			if gMaxMusic > 0 then
				self:PlayMusicList()
			end
		end
	else 
		if gMusicPlaying == true then
			StopMusic()
			gMusicPlaying = false
			self:CancelTimer(self.musicTimer)
		end
	end
end

-- Table Invert for query of key
function table_invert(t)
   local s={}
   for k,v in pairs(t) do
     s[v]=k
   end
   return s
end

-- break appart the musiclist item to find out what file to play in the musicfile table
-- returns a table representing the music info
function MusicInfoFromJukeBoxItem(s)
	local idx = 0
	
	-- get the expansion key
	local b = strfind(s, "/")
	local ns = strsub(s, 1, b-1)
	local xp = GarrisonJukeBox:GetExpanssions() -- 
	local xpi = table_invert(xp)
	idx = xpi[ns]

	-- get music info
	ns = strsub(s, b+1)
	local mt = musictable[idx] -- get table of music infos relative to the expansion
	local t = {}
	
	-- loop through tables to find the music info
	for k, v in ipairs(mt) do
		if v[1] == ns then
			t[1] = v[1]
			t[2] = v[2]
			t[3] = v[3]
			break
		end
	end
	
	return t
end

-- function to play an item in the musiclist table
function GarrisonJukeBox:PlayMusicList()
	gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
	if gMaxMusic > 0 then
		self:CancelTimer(self.musicTimer)
		
		-- check if the list has shrunk since last pass
		if gMusicIndex > gMaxMusic then
			gMusicIndex = gMaxMusic
		end
		
		-- check if the playlist is in shuffle mode
		if GarrisonJukeBox.db.profile.shuffle == true then
			gMusicIndex = math.random(1, gMaxMusic)
		end
		
		-- find what music to play
		local mli = GarrisonJukeBox.db.profile.musiclist[gMusicIndex]
		local info = MusicInfoFromJukeBoxItem(mli) -- gets a table from musictable[n]
		StopMusic()
		PlayMusic(info[2])
		self:Print("Playing: " .. GarrisonJukeBox.db.profile.musiclist[gMusicIndex])
		gMusicPlaying = true
		self.musicTimer = self:ScheduleTimer("TimerEvent", info[3])

		-- increment index after item is playing
		if gMusicIndex == gMaxMusic then
			gMusicIndex = 1
		else
			gMusicIndex = gMusicIndex + 1
		end
	end
end

-- slashcmd
function GarrisonJukeBox:ChatCommand(input)
  -- Open dialog if no parameters, else do something with the parameters
  if not input or input:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory(GarrisonJukeBox.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(GarrisonJukeBox, "gjb", "GarrisonJukeBoxDB", input)
  end
end